#include "weak_classifier.h"

std::vector<mimas::image_ref *> mimas::weak_classifier::image_references;
double mimas::weak_classifier::total_sum_weight_negative_example = 0.;
double mimas::weak_classifier::total_sum_weight_positive_example = 0.;

void mimas::weak_classifier::add_image(image_ref *im_ref)
{
  image_references.push_back(im_ref);
}

void mimas::weak_classifier::normalise_weight_and_calculate_sum(void)
{
  total_sum_weight_positive_example = 0.;
  total_sum_weight_negative_example = 0.;

  for(std::vector<image_ref *>::iterator it = image_references.begin(); it != image_references.end(); it++)
    {      
      //std::cerr<<"image name "<< (*it)->image_name <<"status"<<(*it)->status<<"weight"<< (*it)->weight_associated<<std::endl;
      //!!!do an enum or a define class to avoid risk of mistake
      if((*it)->status == 1)
	total_sum_weight_positive_example += (*it)->weight_associated;
      else
	total_sum_weight_negative_example += (*it)->weight_associated;
    }
  double total_sum = total_sum_weight_negative_example + total_sum_weight_positive_example;

  for(std::vector<image_ref *>::iterator it = image_references.begin(); it != image_references.end(); it++)
    {      
      (*it)->weight_associated /= total_sum;
    }
    
  total_sum_weight_positive_example /= total_sum;
  total_sum_weight_negative_example /= total_sum;
}


void mimas::weak_classifier::determine_threshold(void)
{
  double min = 1.;
  double partial_sum_weight_positive_example = 0.;
  double partial_sum_weight_negative_example = 0.;

  for(std::vector<feature_measure>::iterator it = measures.begin(); it != measures.end(); it++)
    {
      if(it->im_ref->status == 1)
	partial_sum_weight_positive_example += it->im_ref->weight_associated;
      else
	partial_sum_weight_negative_example += it->im_ref->weight_associated;

      double v1 = partial_sum_weight_positive_example + total_sum_weight_negative_example - partial_sum_weight_negative_example;
      double v2 = partial_sum_weight_negative_example + total_sum_weight_positive_example - partial_sum_weight_positive_example;

      if(v1 < min)
	{
	  min = v1;
	  threshold_direction = POSITIVE_ABOVE;
	  threshold = it->result;
	}
      if(v2 < min)
	{
	  min = v2;
	  threshold_direction = POSITIVE_BELOW;
	  threshold = it->result;
	}
    }
  error_measure = min;
}
