//
// $Header: /cvs/mimas2/examples/cornerwise/main.cc,v 1.1.1.1 2005/08/09 15:37:30 engmb Exp $


#include <iostream>
#include <fstream>
#include <GL/glut.h>
#include <vector>
#include <sstream>

#include "canny_tool.h"
#include "cgh.h"
#include "cgh_scene.h"
#include "cgh_wrapper.h"
#include "corner.h"
#include "corner_tool.h"
#include "edge_string_op.h"
#include "filter.h"
#include "image.h"
#include "image_fileinput.h"
#include "image_funcs.h"
#include "mesa_onscreen.h"
#include "prop.h"
#include "x11_window.h"


using namespace std;
using namespace mimas;

class corner_window: public x11_window
{
 public:
  corner_window( x11_display *display, int _width, int _height,
                 const image< int > &_img,
                 vector< cgh_wrapper_ptr > _list ):
    x11_window( display, _width, _height ), img( _img ), list(_list) {}
 protected:
  virtual void paint_event(void)
    throw (mimasexception) {

    mesa_onscreen mesa( this );

    mesa.ortho2D();
      
    glDisable(GL_DITHER);
    glPixelStorei( GL_UNPACK_ALIGNMENT, 1 );
    glPixelZoom( 1, -1 );
    glRasterPos2i( 0, 0 );
    glDrawPixels( width, height,
		  GL_BGRA, GL_UNSIGNED_BYTE, img.rawData().data() );
    glEnable(GL_DITHER);

    mesa.setColor( rgba< unsigned char >( 0, 255, 0 ) );
    
    int c = 0;
    for ( vector< cgh_wrapper_ptr >::const_iterator i = 
	    list.begin(); i!= list.end(); i++, c++ ) {
	
      int
	x = (int)(*i)->position(0),
	y = (int)(*i)->position(1);
	
      glBegin( GL_LINES ); {
	glVertex2i( x - 1, y - 1 );
	glVertex2i( x + 1, y + 1 );
	glVertex2i( x + 1, y - 1 );
	glVertex2i( x - 1, y + 1 );
      }; glEnd();
	
      ostringstream stream; stream << c;
      glPushMatrix(); {
	glTranslated( x + 4, y + 14, 0 );
	glScaled( 0.08, -0.08, 1.0 );
	mesa.text( stream.str() );
      }; glPopMatrix();
	
    };

    mesa.finish();

  };
  
  image< rgba< unsigned char > > img;
  vector< cgh_wrapper_ptr > list;

};

int main(int argc, char **argv)
{
  int retVal = 0;
  glutInit( &argc, argv );
  try {
	image<int> 		im1,im2;
	cgh_scene<int>		view1,view2;

   x11_display display;

	ifstream file1( "bpa1.jpg", ios::binary );
	file1 >> im1;
	cout << "Creating CGHs from first image..." << endl;
   
	view1.createFromImage( im1, 0.1 );
	//    view1.dumpAll( "cgh_" );

	ifstream file2( "bpa2.jpg", ios::binary );
	file2 >> im2;
	cout << "Creating CGHs from second image..." << endl;
	view2.createFromImage( im2, 0.1 );

	//match the histograms
	cgh_wrapper_ptr_list	matchlist;
	cout << "Cross comparing features ..." << endl;

	matchlist= view1.crossCompare( view2 );
   
   vector< cgh_wrapper_ptr > list1, list2;

	for( cgh_wrapper_ptr_list::iterator cghit = matchlist.begin();
        cghit != matchlist.end(); cghit++ ) {
     list1.push_back( (*cghit)             );
     list2.push_back( (*cghit)->best_match );
   };

   corner_window
     win1( &display, im1.getWidth(), im1.getHeight(), im1, list1 ),
     win2( &display, im2.getWidth(), im2.getHeight(), im2, list2 );

	cout << "It finds your spot, irrespective of scale bpa!!" << endl;
	cout << "(image 2 is 50% bigger than image1)" << endl;

   display.event_loop();

  } catch ( exception &e ) {
    cerr << e.what() << endl;
    retVal = 1;
  };
  return retVal;
}


