#include <exception>
#include <iostream>
#include <fstream>
#include <image_fileinput.h>
#include <image_mesaoutput.h>
#include <image_funcs.h>
#include <property_image.h>
#include <image.h>
#include <rgba.h>
using namespace mimas;
using namespace std;
/**
*
*
*
*
* @author Manuel Boissenin
* @date 09/03/2005
*
* Time-stamp: <2005-08-09 11:33:19 engmb>
* Copyright (C) 2005 by Manuel Boissenin
*
 ***********************************************************************/


/*
  Do a query replace
  
  ul: uperLeft
  lr: lowerRight
  w: width
  h: height
  is: iterator_source
  id: iterator_destination
  ixs: line_iterator_source
  ixd: line_iterator_destination
  t: threshold
*/

template<typename ImageIteratorSource>
image<typename ImageIteratorSource::value_type> gradient(ImageIteratorSource ul, ImageIteratorSource lr, unsigned int t)
{  
  typedef typename ImageIteratorSource::value_type pixel_type;
  unsigned int w = lr.x - ul.x, h = lr.y - ul.y;

  image<pixel_type> gradient_image;

  gradient_image.init(w, h);
  
  ImageIteratorSource is(ul);
  ImageIteratorSource id = gradient_image.ul();

  //!!for the moment we don't take into account the border cases
  is.y++,id.y++; 
  for(unsigned int y = 1; y< h - 1; y++, is.y++, id.y++)
    {
      //create an iterator for each line
      ImageIteratorSource ixs(is);
      ImageIteratorSource ixd(id);

      //!!for the moment we don't take into account the border cases
      ixs.x++,ixd.x++;

      for (unsigned int x = 1; x < w - 1; x++, ixs.x++, ixd.x++)
	{		
	  const ImageIteratorSource ref(ixs);

	  int temp1 = ref[Dist2D(0,-1)] - ref[Dist2D(0,1)];
	  int temp2 = ref[Dist2D(-1,0)] - ref[Dist2D(1,0)];
	  	  
	  if(temp1 < 0)
	    temp1 = -temp1;
	   if(temp2 < 0)
	    temp2 = -temp2;

	   if(unsigned(temp1 + temp2) > t)
	     {
	       *(ixd) = pixel_type(255);
	     }
	}      
    }  
  return gradient_image;
}


int main(int argc, char** argv)
{
  int retVal = 0;
  try{
    x11_display display;
    image_mesaoutput< rgba<unsigned char>  > window( &display );
    image_mesaoutput< unsigned char  > window2( &display );

    image<rgba<unsigned char> > colorImage;
    
    std::ifstream file( argv[1], std::ios::binary );
    //file >> image; 
    file>> colorImage;
    /*
    image<rgba<unsigned char> >::ImageIterator i;

    //try different combination
    for (i = colorImage.upperLeft()+ Dist2D(0,10); i.x < colorImage.lowerRight().x; i.x++)
      {
	*i = rgba<unsigned char>(0);
      }

    i = colorImage.ul();

    i[Dist2D(20,5)] = 0;
    i[Dist2D(20,6)] = 0;
    i[Dist2D(20,7)] = 0;
    */
    //window2 << image << mimas::pause();

    window << colorImage << mimas::pause();

    image< int > image( colorImage );
    window2 << gradient( image.ul(), image.lr(), 110 )
	    << mimas::pause();
  } catch (std::exception &e) {
    std::cerr << e.what() << std::endl;
    retVal = 1;
  };
  
  return retVal;
}






