#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/vector.hpp>
#include <boost/numeric/ublas/io.hpp>
#include <iostream>
#include "linalg.h"

using namespace boost::numeric;
using namespace std;
using namespace mimas;

// Parabola.
double f( const double &x )
{
  return 5.0 * x * x - 3.0 * x + 1.0;
}

int main(void)
{
  ublas::matrix< double, ublas::column_major > A( 4, 3 );
  ublas::vector< double > b( 4 );

  // The model is: y = a x^2 + b x + c

  // Record values with some added noise.
  
  // x=-2
  A( 0, 0 ) = -2.0 * -2.0;
  A( 0, 1 ) = -2.0;
  A( 0, 2 ) = 1;
  b( 0 ) = f( -2.0 ) - 0.7;
  
  // x=-1
  A( 1, 0 ) = -1.0 * -1.0;
  A( 1, 1 ) = -1.0;
  A( 1, 2 ) = 1.0;
  b( 1 ) = f( -1.0 ) + 0.05;
  // x=3
  A( 2, 0 ) = 3.0 * 3.0;
  A( 2, 1 ) = 3.0;
  A( 2, 2 ) = 1.0;
  b( 2 ) = f( 3.0 ) + 0.1;
  // x=-5
  A( 3, 0 ) = -5.0 * -5.0;
  A( 3, 1 ) = -5.0;
  A( 3, 2 ) = 1.0;
  b( 3 ) = f( -5.0 ) + 0.44;

  // Apply least square solver.
  ublas::vector< double > m( gels( A, b ) );
  cout << "y = " << m(0) << " x^2 + " << m(1) << " x + " << m(2) << endl
       << "|error| = " << norm_2( prod( A, m ) - b ) << endl;

  return 0;	
}
