#include <boost/numeric/ublas/vector.hpp>
#include <fstream>
#include <QtGui/QPainter>
#include "image_fileinput.h"
#include "linalg.h"
#include "pghDemoWidget.h"

using namespace std;
using namespace mimas;

typedef boost::numeric::ublas::vector< double > Vector;

PghDemoWidget::PghDemoWidget( QWidget *parent, Qt::WFlags f ):
  QFrame( parent, f ), count(1), clipper( new pgh_2dobject )
{
  //(1) create an object -> a square in this case
  //straight line segments would be loaded from ccd extracted lines
  //ordinarily

  // line		*l;
  // int			i;
  image<int> model;

  ifstream modelfile( "images/lgripper0.pgm", ios::binary );
  modelfile >> model;

  clipper->createFromImage(model);
  //clipper->createFromPgmFile("images/lgripper0.pgm","clipper");
#ifndef NDEBUG
  cerr << "Created the Clipper object" << endl;
#endif

  camera_scene.addModel( clipper );
  //camera_scene.addModel("snap03_left_clipper.pgm","clipper");
  //why doesn't this work? -bpa
  /*............................*/

  //export the camera_scene so that the GUI can draw it
    
  //grab an image from the camera
  //extract the edges
  //form straight line segments

  // line_ptr_list *g = new line_ptr_list;

  /*................................................................*/
  //Poly_Lines poly_lines_scene;
  //my_gpoly("snap12_clipper_ROI.pgm", "/tmp/test0.poly", &poly_lines_scene);
	 
  /*................................................................*/

  camera_scene.pgh_match_thresh=0.6;

  // setBackgroundColor( white );		// white background

  startTimer( 0 );
}

void PghDemoWidget::timerEvent( QTimerEvent * )
{
  time_t		starttime,endtime, histtime;
  count++;	
  if (count>15) count=1;
  starttime = time(NULL);

  //camera_scene.updateSceneLines("snap12_clipper_ROI.pgm", 640,480 );
  //count=9;
  char infilename[255];
  sprintf(infilename,"images/snap%.2d.pgm",count);
  ifstream infile( infilename, ios::binary );
  image<int> imagein;
  infile >> imagein;
  camera_scene.updateSceneLines(imagein);
  //camera_scene.updateSceneLines(infilename, 844,576 );
  cerr << endl << infilename << endl;
        
  histtime  = time(NULL);
    
  //export the scene lines to somewhere the GUI can see it!
  //GUI_mono_geom = g;
  //GUI_mono_geom=updateSceneLinesAndReturnLines("snap12_clipper_ROI.pgm", 640,480 );
  // GUI_mono_geom = camera_scene.getListofLinesGUI();
	 
#ifdef COMMENT_OFF
  g=GUI_mono_geom;
  // Dump data to file;
  {
    FILE *tempfile;
    const Vector *tp1, *tp2;
    line_ptr kk;
    line_ptr_list::iterator	k;
		
    tempfile=fopen("outpoints.dat","w");
    for(k=g->begin();k!=g->end();k++ )
		{
        //			tp1=*k->getStart();
        //			tp2=*k->getEnd();
        tp1=((*k)->getStart());
        tp2=((*k)->getEnd());
        //cerr << ((*k)->getStart())->el(0) << endl;
        fprintf(tempfile,"%2.2f %2.2f %2.2f %2.2f\n",(*tp1)(0),(*tp1)(1),(*tp2)(0),(*tp2)(1));
		}
    fclose(tempfile);
  }
#endif
  /*....................................................................*/
#ifndef NDEBUG
  cerr << "Looking for the Clipper ..." << endl;
#endif

	 
	 
  oil = camera_scene.findModel( clipper );
      
  endtime   = time(NULL);
    
  /****Following Code from "gui_qt_eventloop.cc"*****/
  /*To print the Displacement and Orientation of the Object*/
  /*.........................................................................
    object_instance_list **houses = (object_instance_list**) 
    getSymbol("GUI_houses");
    object_instance_list *oil = *houses;
    ..........................................................................*/
  // object_instance_list *oil = oi;

  line_ptr_list_ptr g( new line_ptr_list );
  for ( line_ptr_list::iterator iter=
          camera_scene.getListofLinesGUI()->begin();
        iter!=camera_scene.getListofLinesGUI()->end(); iter++)
	 {
	 	int x1,y1,x2,y2;
		line_ptr	l = *iter;
		const Vector *linetips;
		
		linetips= &l->getStart();
		x1=(int)(*linetips)(0);
		y1=(int)(*linetips)(1);

		linetips= &l->getEnd();
		x2=(int)(*linetips)(0);
		y2=(int)(*linetips)(1);

		line_ptr pl( new line(x1,y1,x2,y2) );
		g->push_back(pl);

	 }

  if ( oil ) {

    for ( coordinate_xform_ptr_list::iterator it = oil->getList().begin();
          it != oil->getList().end(); it++ )
      {
        coordinate_xform_ptr x = *it;
        
        double xdisp, ydisp;
        Vector d( x->getDisplacement() );
        double ang = (double)x->getRotation();
        
        xdisp = d(0);
        ydisp = d(1);
        
        cout << "The Displacement = " << xdisp << "," << ydisp << " and Orientation " << ang << endl;
        
        for ( line_ptr_list::iterator iter=
                clipper->getListofLines()->begin();
              iter!=clipper->getListofLines()->end(); iter++ )
          {
            line_ptr l = *iter;
            const Vector *linetips;
          
            Vector s(2),e(2),shift(2);
            angle a;
          
          
            linetips= &l->getStart();
            s(0)=(int)(*linetips)(0);
            s(1)=(int)(*linetips)(1);

            linetips= &l->getEnd();
            e(0)=(int)(*linetips)(0);
            e(1)=(int)(*linetips)(1);
				
            a= angle(ang);
            s = rotate( s, -a );
            e = rotate( e, -a );
				
            //shift=vector(xdisp-(clipper.getWidth()/2),ydisp-(clipper.getHeight()/2));
            shift(0) = xdisp;
            shift(1) = ydisp;
            s = s + shift;
            e = e + shift;
            line_ptr pl( new line(s,e) );
            g->push_back(pl);

          }

      }/*for (it = xform_list->begin(); it != xform_list->end(); it++) */
    /****Above Code from "gui_qt_eventloop.cc"*****/

    // GUI_mono_geom=g;
  };

  cout << "total search time = " << endtime-starttime << "seconds " << endl;
  cout << "( " << histtime - starttime << " seconds to generate hists for the 104 scene lines " << endl;
  cout << "  and " << endtime - histtime << " seconds to compare and locate the objects. " << endl;
  cout << "------------------------------------------------------------------" << endl << endl;

  update();
}

void PghDemoWidget::paintEvent( QPaintEvent * )
{
  QPainter paint( this );

  //draw it!
  line_ptr_list_ptr l = camera_scene.getListofLinesGUI();
  if ( l != NULL ) {
    paint.setPen( Qt::black );

    for( line_ptr_list::iterator i=l->begin(); i!=l->end(); i++ )
      {
        line_ptr ln = (*i);
        const Vector *start, *end;
		
        start = &ln->getStart();
        end   = &ln->getEnd();
		
        paint.drawLine( (int)(*start)(0), (int)(*start)(1),
                        (int)(*end)(0), (int)(*end)(1) );
      }	

    if ( oil ) {
      //I can't be bothered to draw the actual shapes... I'll just mark the position
      //and orientation for now
      for( coordinate_xform_ptr_list::iterator it=oil->getList().begin();
           it!=oil->getList().end(); it++)
        {
          coordinate_xform_ptr x = *it;
	    
          double	x1,x2,y1,y2;
          Vector d( x->getDisplacement() );
          double	a = (double)x->getRotation();

          //paint.setPen(Qt::red);
          paint.setPen(Qt::blue);
          x1 = d(0) + cos( a ) * 10;
          y1 = d(1) + sin( a ) * 10;
          x2 = d(0) - cos( a ) * 10;
          y2 = d(1) - sin( a ) * 10;
          paint.drawLine( (int) x1, (int) y1, (int)x2, (int) y2 );
          paint.setPen(Qt::blue);
          paint.drawLine( (int) x1, (int) y1, (int)x1, (int) y1 );
        }
    }
  }
}
