// #define TESTGREYSCALE

#include <exception>
#include <GL/glut.h>
#include <iomanip>
#include <iostream>
#include <popt.h>
#include <sstream>
#include "image_mesaoutput.h"
#include "image_filesinput.h"
#include "image_fileoutput.h"
#include "image_v4linput.h"

using namespace boost;
using namespace mimas;
using namespace std;

#ifdef TESTGREYSCALE
typedef unsigned char pixel;
#else
typedef rgba< unsigned char > pixel;
#endif

int main( int argc, char *argv[] )
{
  int
    retVal = 0,
    savePics = 0,
    channel = 0,
    brightness = 32768,
    contrast = 32768,
    count = 0,
    width = -1,
    height = -1,
    shutter = -1,
    gain = -1,
    balance = -1;

  const char *device = NULL;

  // Initialise GLUT.
  glutInit( &argc, argv );

  struct poptOption optionsTable[] = {
    { "brightness", '\0', POPT_ARG_INT | POPT_ARGFLAG_SHOW_DEFAULT,
      &brightness, 0, "Brightness for video4linux-input [0..65535]",
      "brightness" },
    { "contrast", '\0', POPT_ARG_INT | POPT_ARGFLAG_SHOW_DEFAULT,
      &contrast, 0, "Contrast for video4linux-input [0..65535]", "contrast" },
    { "balance", '\0', POPT_ARG_INT | POPT_ARGFLAG_SHOW_DEFAULT,
      &balance, 0, "White balance for video1394-input", "balance" },
    { "device", '\0', POPT_ARG_STRING,
      &device, 0, "Specify video4linux/video1394-device", "device-name" },
    { "channel", '\0', POPT_ARG_INT | POPT_ARGFLAG_SHOW_DEFAULT,
      &channel, 0, "Specify video4linux-channel/video1394-node",
      "channel-number" },
    { "width", 'w', POPT_ARG_INT,
      &width, 0, "Force width of camera-image", "width in pixel" },
    { "height", 'h', POPT_ARG_INT,
      &height, 0, "Force height of camera-image", "height in pixel" },
    { "savepics", '\0', POPT_ARG_VAL, &savePics, 1,
      "Save pictures for debugging-purposes (edge*.ppm)", 0 },
    POPT_AUTOHELP
    { NULL, 0, 0, NULL, 0 } };

  poptContext optCon( poptGetContext( NULL, argc, (const char **)argv,
                                      optionsTable, 0 ) );
  poptSetOtherOptionHelp( optCon, "[FILE]..." );

  try {

    // Parse command-line.
    char c;
    while ( ( c = poptGetNextOpt( optCon ) ) >= 0 );

    const char *strayParameter = poptGetArg( optCon );
    MMERROR( strayParameter == NULL, mimasexception, ,
             "Unexpected command-line argument \"" << strayParameter
             << "\"" );
    // Throw exception on error.
    MMERROR( c >= -1, mimasexception, ,
             poptBadOption( optCon, POPT_BADOPTION_NOALIAS ) << ": "
             << poptStrerror( c ) );

    image_v4linput< pixel > imageInput
      ( device == NULL ? "/dev/video": device, channel,
        width, height );

    MMERROR( brightness >= 0 && brightness <= 65535, mimasexception, ,
             "Brightness must be in [0..65535], but was " << brightness
             << "." );
    MMERROR( contrast >= 0 && contrast <= 65535, mimasexception, ,
             "Contrast must be in [0..65535], but was " << contrast
             << "." );

    imageInput.setSensivity( brightness, 32768, 32768, contrast );

    x11_display display;

    image_mesaoutput< pixel > win( &display );
    
    while ( true ) {

      image< pixel > image;
      MMERROR( imageInput >> image, mimasexception, ,
               "Error retrieving image." );

#ifndef NDEBUG
      cerr << '*' << flush;
#endif

      win << image;

      if ( savePics ) {

        ostringstream outputFileName;
        outputFileName << "cam" << setw( 6 ) << setfill( '0' )
                       << count << resetiosflags( ios::floatfield )
                       << ".ppm";
       
        // Use fast PPM output. 
        ofstream ofile( outputFileName.str().c_str(), ios::binary );
        MMERROR( ofile << setformat( "PPM" ) << image,
                 mimasexception, ,
                 "Error writing to file \""
                 << outputFileName.str().c_str() << '\"' );

        count++;
        
      };

    };

#ifndef NDEBUG
    cerr << endl;
#endif

  } catch ( std::exception &e ) {

    cerr << e.what() << endl;
    // Display command-line information.
    poptPrintUsage( optCon, stderr, 0 );
    retVal = 1;

  };

  // Call destructor of command-line parser.
  poptFreeContext( optCon );

  return retVal;
}
