//
// canny stuff. rewritten, though based on tina
// stumeikle Sat May  4 19:22:36 2002
//
//
//




#ifndef FILTER_CANNY_H
#define FILTER_CANNY_H

#include <cmath>
#include "filter.h"
#include "gauss.h"
#include "image.h"
#include "image_op.h"
#include "property_image.h"
#include "edge_string.h"
#include "edge_string_op.h"

namespace mimas {
	
/** Canny edge extraction tool.
    Algorithms and inspiration from TINA.
    \c canny_tool is an operator which is used to extract edge strings from
    input images. Edge extraction method is based upon Canny's algorithm and
    includes Canny's post processing methods. Found edges are grouped into
    continuous strings. This class is a template class because it must operator
    on a template class (the input image).


    There are some tutorials about the Canny edge detection technique here:
    \li http://www.pages.drexel.edu/~weg22/can_tut.html and here,
    \li http://www.cee.hw.ac.uk/hipr/html/canny.html and here,
    \li http://homepages.inf.ed.ac.uk/rbf/HIPR2/canny.htm
    
    @author Stuart Meikle (stu@stumeikle.org)
    @date Sat May  4 19:22:36 2002
    @see edge_string */
template<typename T>
class	canny_tool
{
    private:
	double	sigma;
	double	precision;
	double	low_edge_thres;
	double	up_edge_thres;
	int	len_thres;

    //operator. operates on images
    public:

    canny_tool(void)		///< constructor. default prams should be ok for most tasks.
    {
	sigma = 1.0;
	precision = 0.01;
	low_edge_thres = 3.0;
	up_edge_thres = 5.0;
	len_thres = 5;
    }
    /** Perform edge-detection.
	Find the edges in the given input image and return them as a list of
	edge strings.
	@see edge_string */
    edge_string_ptr_list findEdgeStrings( const image<T> &input ) 
    {
      image<int>
        // Compute horizontal and vertical gradients.
        grad_x( gaussGradientX< float >( input, sigma ) ),
        grad_y( gaussGradientY< float >( input, sigma ) ),
	// Create grad sq. The following would be very elegant, but more memory
        // intensive:
	// gradsq = (grad_x*grad_x) + (grad_y*grad_y);
	// (... so we wont ...)
        gradsq( sumSquares( grad_x, grad_y ) );

	// Do non maxima suppression
	// (errr.. shouldnt we look for zero crossings of grad^2 im?)
	edge_image edge_im;
	filter<int>::nonMaximaSuppression( grad_x, grad_y, gradsq,
                                           40.0, edge_im );
	grad_x.clear();
	grad_y.clear();
	gradsq.clear();

	//(5.0) post process the resulting edgels
	// what's the best way to do this ? we could create an edge_image function
	// but it would be better *not* to have specific edge images. better to have image properties
	// than specific image types. anyway image<int> is a perfectly valid edge image too.
	edge_string_ptr_list edge_strings;
	edge_string_op	eop;

	edge_strings = eop.extractFromImage( edge_im, 5, 5.0 );

	//done canny ----------------------------------------------------
	return edge_strings;
    }
};

};

#endif
