#include <csignal>
#include <cstdlib>
#include <fcntl.h>
#include <sstream>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

namespace mimas {


template< typename T >
image_mplayeroutput< T >::image_mplayeroutput( int _width, int _height )
  throw (mimasexception):
  width(_width), height(_height)
{
  // Compute unique filename for pipe.
  const char *tmppath = getenv("TMP");
  std::ostringstream pipenameStream;
  pipenameStream << ( tmppath != NULL ? tmppath : "/tmp" )
                 << "/mimas-display-pipe.pid" << (int)getpid() << ".XXXXXX";
  pipename = pipenameStream.str();

  // Try to create file.
  MMERROR( mkstemp( (char *)pipename.c_str() ) != -1, mimasexception, ,
           "Could not create temporary file \"" << pipename << "\"." );
  // Remove it afterwards.
  remove( pipename.c_str() );

  // create fifo
  mkfifo( pipename.c_str(), 0600 );
  // int	status=mkfifo(pipename.str().c_str(),S_IWUSR | S_IRUSR);
  
  // open image pipe
  imgpipe = open( pipename.c_str(), O_RDWR );
  // imgpipe= open(pipename.str().c_str(),O_RDWR | O_DIRECT);

  // Using rgb2bgr=swap, because mimas::yv12_to_rgba is really doing the
  // inverse of mimas::rgba_to_yv12, which again has been tested with libxine
  // and video4linux. There seems to be an error in mplayer or X11.
  std::ostringstream cmd;
  cmd << "mplayer -nofs -slave -nocache -rawvideo on:w=" << width
      << ":h=" << height << ":format=yv12 -vop rgb2bgr=swap -vo x11 -ao null " << pipename
      << " &> /dev/null";

  // If you still wish to encode then uncomment the following line. Note
  // that the display will not stop, since current versions of mencoder
  // does not support the slave option. So press CTRL-C to stop the program
  // and don't forget to killall -9 mencode. Also remove the pipe called
  // img?????? from /tmp
  // sprintf(cmd,"mencoder -rawvideo on:w=%d:h=%d %s -ovc lavc -lavcopts vbitrate=4 -ofps 10 -oac mp3lame -lameopts vbr=2:aq=0:mode=3:aq=9 -o %s &> /dev/null",w,h,pipename, avifn);
  
  // Provision for Mplayer 1.0:
  // sprintf(cmd,"mencoder -slave -rawvideo on:w=%d:h=%d %s -ovc lavc -lavcopts vbitrate=4 -ofps 10 -oac mp3lame -lameopts vbr=2:aq=0:mode=3:aq=9 -o %s &> /dev/null",w,h,pipename, avifn);

  mplayer = popen( cmd.str().c_str(), "w" );

  if ( mplayer == 0 ) {
    close( imgpipe );
    remove( pipename.c_str() );
    MMERROR( false, mimasexception, ,
             "image_mplayeroutput cannot open mplayer");
  };

  // no caching of instructions
  setvbuf( mplayer, NULL, _IOLBF, 0 );
}

template< typename T >
image_mplayeroutput< T >::~image_mplayeroutput(void)
{
	// send quit command to mplayer
	fprintf(mplayer,"\nquit\n");
	fflush(mplayer);

	// send a single frame to the fifo pipe - otherwise mplayer won't respond
   for ( int i=0; i<width*height/2; i++ )
     ::write(imgpipe,"\0\0\0",3);
#ifndef NDEBUG
   std::cerr << "synching pipe" << std::endl;
#endif
	fsync(imgpipe);

	//close the mplayer program pipe
#ifndef NDEBUG
	std::cerr << "closing pipe" << std::endl;
#endif
	pclose(mplayer);
#ifndef NDEBUG
	std::cerr << "pipe closed" << std::endl;
#endif
	
	//close the image pipe
	close( imgpipe );
	remove( pipename.c_str() );
}

};
