#include <cmath>
#include <iostream>
#include <fstream>
#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/vector.hpp>
#include <boost/numeric/ublas/io.hpp>
#include "object.h"
#include "image.h"

#ifndef MOMENTS_TOOL_H
#define MOMENTS_TOOL_H

namespace mimas {
	/**
	 * Computation of image moments and Hu invariant moments
	 * @author Bala Amavasai (bala@amavasai.org)
	 * @date Tue Apr 12 16:00:05 BST 2005
	 * $Header: /cvs/mimas2/include/moments_tool.h,v 1.1.1.1 2005/08/09 15:37:45 engmb Exp $
	 */

	class moments_tool: public object
	{
		public:
			typedef boost::numeric::ublas::matrix< double > Matrix;
			typedef boost::numeric::ublas::vector< double > Vector;

		private:


		public:
			moments_tool(void); ///< constructor
			boost::numeric::ublas::matrix< double > findMoments(const image< unsigned char > imagein, const unsigned char bglevel=0, bool binaryImage=false); ///< find image moments and return a 5x5 matrix containing the moments. The bglevel parameter is the background grey level that is ignored when find the moment
			boost::numeric::ublas::vector< double > findHuInvariantMoments(const image< unsigned char > imagein, const unsigned char bglevel=0, bool binaryImage=false); ///< find the 7 Hu invariant moments and return it as a vector. Note that the vector indices from 0..6, so Hu moment 1 is 0, Hu moment 2 is 1, etc. The bglevel parameter sets the background grey level so that these pixels are ignored in the image.


	};

	
}

#endif
