//
// pairwise histograms
// stuart meikle
// Tue Feb 22 11:57:51 2000
//


// lets define some structure here:
// a hough transform is a form of image
// a pairwise_gh is a form of hough transform

#ifndef PGH_H
#define PGH_H

#include <boost/numeric/ublas/vector.hpp>
#include <boost/smart_ptr.hpp>
#include "erf.h"
#include "hough_transform.h"
#include "line.h"
#include "object.h"
#include "pgh_type.h"
#include <list>

//check:
//all uses of pair_dist and pair_angle are modified
//compare lines filters out window_r stuff properly

namespace mimas {
/** Pairwise geometric histograms.
    Pairwise geometric histograms. see the many papers by Evans,Thacker,Ashbrook and Caparrelli
    This flaky implementation by stuart meikle 22.02.2000, based on original Tina tool source code.

    A pairwise geometric histogram is a special kind of hough transform. Much
    of the code here was taken from Tina, and modified to cope with the new
    base classes. Its easy enough to use, just do
    \code
    pgh	my_pgh;
    my_pgh.initialise(); my_pgh.describeLine( ref_line, set_of_lines );
    \endcode
    which will create a pgh for that line. You should add extra code to set
    the parameters used during creation. There probably shouldn't be an erf
    array allocated for every Pgh ! oops. To fix this make the variable
    instance a static so that only one is used for all instances of \c pgh.
    Also please note that the 'directed' kind of histogram is now formed
    correctly unlike in Tina. Also i think there should be a warning somewher
    ein the creation process to say : Hang about mate, you've given me an
    object which is 500x500 and you want me to use histograms with a max
    distance axis of 50? Ie the histogram should scale with the model. Because
    the pgh is an image type, to do a bhattacharyya comparison between two
    do:
    \code
    pgh p1,p2;
    // ... enter values into p1 and p2 ...
    p1.sqr_root_and_normalize(); 
    p2.sqr_root_and_normalize(); 
    result =p1.dotProduct(p2);
    \endcode */
class	pgh	: public hough_transform
{
    private:
	//pairwise specific parameters
	mm_erf		erfarray; ///< look up table for integral(erf(x)) which we cannot calculate directly
				  // I guess we don't need one of these
			          // for EVERY pairwise !! FIX!!!!@

    protected:

	//pairwise specific params
	pgh_type	pairs_type; ///< specifies the encoding used to create the histogram
	double		dbin_max, dbin_min; ///< distance range which the histogram spans
	double		dbin_size, dist_ramp; ///< range of each histogram bin wrt distance, number of angle bins in hist
	int			num_abin;
	double		angle_sigma, abin_size; ///< range of each angle bin, sigma= angle m'ment error estimate
	double		*pair_dist, *real_pair_dist; ///< ??
	double		*pair_angle,*real_pair_angle; /// < ??


    public:
    typedef boost::numeric::ublas::vector< double > Vector;
	pgh(void);
	~pgh(void);
	//typical pairwise functions

	// added by fbc, 11.10.01
	void	initialise(	pgh_type newpairs_type, double newdbin_max, double newdbin_size,
						double newdist_ramp, int newnum_abin, double newangle_sigma);

	void	initialise( void );

	void	describeLine( const line &ref_line, const line_ptr_list &l);

	void	init_pairs_entry(void);

	bool    adjust_for_distance_ramp( 	double *weight,  double *min_d,
					    				double *max_d,double dbin_min,
					    				double dist_ramp );

	void	build_normalized_pairwise(	const line &ref_line,
                                    const line_ptr_list& geom );

	void 	compare_lines(const line &refline, const line &line);

	void 	compare_lines_mirror(const line &l1, const line &l2);

	void	make_entry_mirror(	double min_d,double max_d,
					    		double angle,double weight);

	void 	make_entry_rotate(	double min_d, double max_d,
								double angle, double weight);

	void	compare_lines_rotate( const line &l1, const line &l2);

	void 	make_entry_directed(double min_d, double max_d,
                    	    	double angle, double weight);

	void 	make_entry_direct(	double min_d, double max_d,
								double angle, double weight);

	void	compare_lines_directed(	const line &l1, const line &l2);

	Vector dir_vec( const Vector &isct, const Vector &p1,
                   const Vector &p2 );

	void	get_pdist(	double *pdist,double pdmin,
						double pdmax, double bin_width,
						double ramp,  int *low_bin,
						int *high_bin);

	void 	sqr_root_and_normalize(void);
};

///
typedef boost::shared_ptr< pgh > pgh_ptr;

///
typedef std::list< pgh_ptr > pgh_ptr_list;

}

#endif

