#ifndef SLAP_TOOL_H
#define SLAP_TOOL_H

#include <boost/numeric/ublas/vector.hpp>
#include <boost/shared_ptr.hpp>
#include <cmath>
#include "line.h"
#include "image.h"
#include "edge_string.h"
#include "edge_string_op.h"

namespace mimas {
/**
 *
 * SLAP == Straight Line APproximation tool.
 *
 * use this tool to turn edge strings into lists of straight line segments.
 * Algorithm here is based on the original by Dr xxxx (can fbc provide this 
 * reference please?). Implementation by stu. Sun 13 Oct 2002 14:45:26 CEST
 *
 *
 * Algorithm takes a parameter- anglesigma which is used to determine how the
 * edge strings should be broken into straight line segments. A high value 
 * means that only large changes in the angle of an edge string will be 
 * encorporated into a single straight line, therefore producing a very 
 * approximate set of output lines (blocky). With a lower value the straight
 * lines more closely follow the edge strings and the output will look more
 * like the input. (smoother)
 *
 *
 * @author$
 * @date$
 * $Header: /cvs/mimas2/include/slap_tool.h,v 1.1.1.1 2005/08/09 15:37:45 engmb Exp $
 */
class	slap_tool	///< A tool. Not derived from anything.
{
    private:
	double		anglesigma;///< what angle differences can be 'detected' by this tool? see above
			//use this to simplify parameter lists

    public:
    typedef		string_el_ptr_list::iterator string_it;
    typedef boost::numeric::ublas::vector< double > Vector;

    /** 
    * Default constructor. Sets a default anglesigma value. 
    * NOTE: this value needs to be small otherwise the algorithm fails
    *     : as this implementation requires that tan(anglesigma) is 
    *     : approx= anglesigma
    */
    slap_tool(void)
    {
	anglesigma = M_PI/32.0;	
    }
    
    slap_tool( double d )
    {
	anglesigma = d;
    }

    /**
    * Return a list of straight lines from a given list of edge strings. 
    * A suitable list of edge strings is produced by, for example, the 
    * canny_tool.
    */
    line_ptr_list findStraightLines( edge_string_ptr_list& es_list );
    
    private:
    //the rest of the functions are internal to the tool, and shouldn't need to be
    //touched i imagine.
    line_ptr_list poly_strings( edge_string_ptr_list& es_list, double thres );
    line_ptr_list poly_string( edge_string& es , double thres );
    void		poly_prox( string_it start_it, string_it end_it, double thres, 
				    line_ptr_list& result );

};
};

#endif
