#ifndef TIMER_H
#define TIMER_H

#include <boost/shared_ptr.hpp>
#include <sys/time.h>
#include "mimasconfig.h"
#include "object.h"

namespace mimas {

/// Class for measuring time.    
class timer
{
 public:
  /** Constructor starting timer.
      The constructor will call \c reset() to. */
  timer(void) { reset(); }
  /** Restart timer.
      Reset the time elapsed to zero again. */
  void reset(void) {
#ifdef HAVE_GETTIMEOFDAY
    gettimeofday( &time, NULL );
#else
    assert( false );
#endif
  }
  /** Get elapsed time.
      The time elapsed (in seconds) is returned.
      @return Time elapsed since last call of \c timer or \c reset() . */
  double elapsed(void) const;
 protected:
  /// Time of construction of this object.
  struct timeval time;
};

///
typedef boost::shared_ptr< timer > timer_ptr;

};


#endif
