#include "nlinalg.h"

#ifdef HAVE_LIBMINPACK

#include <boost/numeric/ublas/vector.hpp>

extern "C" {
#include <minpack.h>
}

#include "mimasexception.h"

namespace mimas {

// lmdif minpack invocation.
lmdif_t::Vector lmdif_t::operator()
     (const Vector &_A,
      void (*fnc)(int *m, int *n, double *x, double *fvec, int *iflag),
      int nb, int maxfev, double tolerance, int mode, int factor, 
      double epsfcn) const
{
  // make a working copy
  Vector A( _A );

  int 
    M = nb,
    N = A.size(),
    MAXFEV = maxfev, //200*(N + 1)
    INFO,
    MODE = mode,
    NPRINT = 0,
    NFEV, 
    LDFJAC = M,
    *IPVT = new int[N];

  double 
    FTOL = tolerance,
    XTOL = tolerance,
    GTOL = 0.0,
    FACTOR = factor,
    EPSFCN = epsfcn,
    *FJAC = new double[M*N],
    *DIAG = new double[N],
    *FVECT = new double[M],
    *WA = new double[M + 3*N],
    *QTF = new double[N];

  // check the size
  assert( M >= N );

  // call to the lmdif_ minpack function
  lmdif_(fnc, &M, &N, A.data().begin(), FVECT, &FTOL, &XTOL,
	 &GTOL, &MAXFEV, &EPSFCN, DIAG, &MODE, &FACTOR, &NPRINT,
	 &INFO, &NFEV, FJAC, &LDFJAC, IPVT, QTF, 
	 &WA[0], &WA[N], &WA[2*N], &WA[3*N]);

  delete[] FVECT;
  delete[] IPVT;
  delete[] FJAC;
  delete[] DIAG;
  delete[] WA;
  delete[] QTF;

  MMERROR( INFO >= 1 && INFO <= 3, mimasexception, ,
           "lmdif: error on INFO output : " << INFO << "." );

  return A;
}

// lmdif1 minpack invocation.
lmdif1_t::Vector lmdif1_t::operator()
     (const Vector &_A,
      void (*fnc)(int *m, int *n, double *x, double *fvec, int *iflag),
      int nb, double tolerance) const
{
  // make a working copy
  Vector A( _A );

  int 
    M = nb,
    N = A.size(),
    INFO,
    LWA = M*N + 5*N + M,
    *IWA = new int[N];

  double 
    TOL = tolerance,
    *FVECT = new double[M],
    *WA = new double[LWA];

  // check the size
  assert( M >= N );

  // call to the minpack function
  // lmdif1_ is a simplified version of lmdif_
  lmdif1_(fnc, &M, &N, A.data().begin(), FVECT, &TOL, &INFO,
	  IWA, WA, &LWA);

  delete[] IWA;
  delete[] FVECT;
  delete[] WA;

  MMERROR( INFO >= 1 && INFO <= 3, mimasexception, ,
           "lmdif: error on INFO output : " << INFO << "." );

  return A;
}

};

#endif

