/**
 *
 * adding scale invariant recognition capabilities
 * to the POL stuff.
 *
 * (again)
 * stumeikle 20030228
 *
 */

#include "mimasconfig.h"
#include <cstdarg>
#include "linalg.h"
#include "line.h"
#include "pol_match.h"
#include "pol_locator_scale.h"

namespace mimas {
using namespace std;

void		pol_locator_scale::init(int numdims,...)
{
		va_list	vlist;
		va_start(vlist, numdims );

		//get whatever parameters we want. in this case the image width and height
		//and the sigma and precision of the gauss.
		int	w,h;
		double	s,p;

		w=va_arg(vlist, int);
		h=va_arg(vlist, int);
		s=va_arg(vlist, double);
		p=va_arg(vlist, double);

		//allocate aaanother big image for scale stuff
		//!this image is actually totally unnecessary for recognition
		//!but we need it in order to return an estimate of the scale 
		//!to the user.
		scale_im.init(w,h);

		va_end(vlist);

		pol_locator_hough::init( numdims, w,h,s,p );
}

 void		pol_locator_scale::addEntry( double weight, ... )
 {
		va_list	vlist;
		va_start( vlist, weight );

      Vector v( 2 );
      v( 0 ) = va_arg(vlist, double)+ox;
      v( 1 ) = va_arg(vlist, double)+oy;
		double		a = va_arg(vlist, double);
      Vector sv( 2 );
      sv( 0 ) = va_arg(vlist, double)+ox;
      sv( 1 ) = va_arg(vlist, double)+oy;
		double		min_s=va_arg(vlist, double);
		double		max_s=va_arg(vlist, double);

		/* prototype...increase weight
		double		t = tan(a);
		if (t<0)	t=-t;
		if (t>1)	t=1/t;
		weight = weight*(1+t);
		*/

		//we can now make a series of entries to cover all scale possibilities
		//(meaning a line...)
      Vector delta( v - sv );
		double model_scale = norm_2( delta );
      Vector dmin( 2 ),dmax( 2 ), v1( 2 ), v2( 2 );

		dmin = delta*min_s;
		dmax = delta*max_s;
		//v1 = sv + dmin;
		//v2 = sv + dmax;
		v1 = dmin;
		v2 = dmax;

//cout << "min_scale=" << min_scale << "    max_scale=" << max_scale << endl;
//cout << "line = " << v1.el(0) << "x" << v1.el(1) << " to " << v2.el(0) << "x" << v2.el(1) << endl;

		line		l( v1, v2 );

		//now we have to draw the line point by point, in order to get the weights
		//right
		// double		len = l.mag();
		// double		i;
		Vector deltav( unit( l.getVector() ) );

		double		vlen = norm_2( v1 );
		double		vlenend = norm_2( v2 );
		double		w,scale;
		int		ix,iy;
		pol_scale_info	si;

		v1+=sv;
		for(;vlen<vlenend;vlen+=1.0)
		{
		    //plot a point at 'v1'
		    scale = vlen / model_scale;
		    w = weight/(scale*scale);

		    addGaussPointFast(v1, w, a);
		    ix=(int)v1(0);	iy=(int)v1(1);

		    si = scale_im.getPixel(ix,iy);
		    si.weight+=w;
		    si.scale+=scale*w;
		    scale_im.setPixel(ix, iy, si);

		    v1+=deltav;
		}

		va_end(vlist);
 }


 /**
  * use an almost-copy of the parent function and include an
  * estimate of the scale. there are better ways to go about this!
  * but this will be ok for now.
  *
  */
 	pol_match_ptr_list	pol_locator_scale::getMatches()
	{
	    //process the now full hough and return a list of the found objects and their
	    //positions
	    pol_match_ptr_list	retval;

	    //display();

	    int	x,y;
	    //(1) threshold
	    int	cutoff = (int)(thres*thres);
	    for(y=0;y<getHeight();++y)
	    {
		for(x=0;x<getWidth();++x)
		{
		    if (pixel(x,y).magSquared() < cutoff)
		    {
			pixel(x,y).x=0;
			pixel(x,y).y=0;
		    }
		}
	    }
	   // display();

	    image<int>	im;
	    im.init( getWidth(), getHeight() );

	    //(2) find the peaks
	    for(y=1;y<getHeight()-1;++y)
	    {
		for(x=1;x<getWidth()-1;++x)
		{
		    int	v = pixel(x,y  ).magSquared();

		    if (v==0)
			continue;

		    int vn= pixel(x,y-1).magSquared();
		    int vs= pixel(x,y+1).magSquared();
		    int ve= pixel(x+1,y).magSquared();
		    int vw= pixel(x-1,y).magSquared();
		    int vne= pixel(x+1,y-1).magSquared();
		    int vnw= pixel(x-1,y-1).magSquared();
		    int vse= pixel(x+1,y+1).magSquared();
		    int vsw= pixel(x-1,y+1).magSquared();

		    if (v>=vn && v>=vs && v>=ve && v>=vw &&
		        v>=vne&& v>=vnw&& v>=vse&& v>=vsw )
		    {
		    	im.setPixel(x,y,255);

			//ok got a peak.
         pol_match_ptr o( new pol_match );
         Vector pos( 2 );

         pos( 0 ) = (double)x+0.5; pos( 1 ) = (double)y+0.5;
			o->setPosition(pos);

			angle		a(  (double)pixel(x,y).y,
						    (double)pixel(x,y).x );

			o->setOrientation( a );

#ifndef NDEBUG
			cerr << "Found an instance at ( " << pos(0) << "x"
              << pos(1)
              << " with angle " << (double)a/0.0174533
			     << " strength " << sqrt((double)v);
#endif

			pol_scale_info si= scale_im.pixel(x,y);
#ifndef NDEBUG
			cerr << "and scale " << si.scale/si.weight <<endl;
#endif

			o->setScale(si.scale/si.weight);
			o->setStrength(sqrt((double)v));
			retval.push_back(o);

			//blank the neighbourhood in case its a blurry point.
			//no don't do this. need another hough really.
			//for(int j=y-1;j<=y+2;j++)
			//{
	    	 	//    for(int i=x-1;i<=x+2;i++)
			//	setPixel(i,j,0);
	    		//}

    		    }//if peak
		}
	    }

	 //   im.display();

	    return retval;
	}
}

