/**
*
* mm pol model.cc
* stumeikle Sun 27 Oct 2002 22:21:55 CET
*
* see header file for comments.
* $Header: /cvs/mimas2/lib/pol_model.cc,v 1.1.1.1 2005/08/09 15:37:29 engmb Exp $
*
*/

#include <boost/shared_ptr.hpp>
#include <fstream>
#include "linalg.h"
#include "image.h"
#include "image_fileinput.h"
#include "pol_model.h"
#include "pol_locator.h"
#include "canny_tool.h"
#include "edge_string.h"
#include "slap_tool.h"
#include "line.h"

namespace mimas {
  using namespace std;
  using namespace boost;

  bool	pol_model::scaleInvariant()
  { return scale_inv; }

  void	pol_model::createFromImageFile( const char* fn )
  {
    image<int>			im1;
    canny_tool<int>		canny;
    edge_string_ptr_list		edge_strings;
    slap_tool			slapper(M_PI/num_abins);

    ifstream file( fn, ios::binary );
    file >> im1;
    edge_strings=canny.findEdgeStrings( im1 );

    model_lines=slapper.findStraightLines( edge_strings );

    //delete the edge strings
    // edge_string_ptr_list::iterator	sit;
    // for(sit=edge_strings.begin();sit!=edge_strings.end();++sit)
    // delete (*sit);
    edge_strings.clear();

    // double			model_rad;
    int			model_num_lines=0;
    Vector centroid(2);

    for( line_ptr_list::iterator
	   line_it=model_lines.begin();
	 line_it != model_lines.end(); line_it++ )
      {
	line_ptr l = *line_it;

	// we need to exclude short lines here otherwise regular shapes
	// can have centroids which are not in their centre, which messes
	// up their recognition strengths
	if (l->length()<min_len)
	  continue;

	centroid+= l->getStart();
	centroid+= l->getEnd();

	model_num_lines++;
      }

    centroid=centroid/(double)(model_num_lines*2);

    //find the max radius and move all the lines to new origin
    double	maxrad=0;
    for(line_ptr_list::iterator
	  line_it=model_lines.begin();
	line_it!=model_lines.end();line_it++)
      {
	line_ptr l = *line_it;
	Vector *start,*end,t;

	start = &l->getStart();
	end   = &l->getEnd();

	*start = *start - centroid;
	*end   = *end   - centroid;
	if ( norm_2( *start ) > maxrad )
	  maxrad = norm_2( *start );
	if ( norm_2( *end ) > maxrad )
	  maxrad = norm_2( *end );
      }

    if (!scale_inv)
      setRadius(maxrad);
    else
      setRadius(max_scale*maxrad);

    createFromLines(model_lines);
  }

  void	pol_model::createFromLines( line_ptr_list &slaps )
  {
    //lines should be arranged about a centroid

    //algorithm
    //(1)go through the straight lines, take every conceivable pair of
    //lines
    //	take a and b  as well as  b and a, but not a and a or similar
    //(1.5) throw away lines which are too short. (ie a or b < min len)
    //(2) calculate the angle between them and find the nearest suitable
    //bin (ie map 0-2M_PI to 0-num_abins).
    //(3) calculate the remainder of the required properties and then
    //add the structure to the relevant hash bin list.
    angle	a;
    double		da;
    int		abin;
    double		len1,t;
    // double len2;
    double		reallen2;
    Vector cross(2),tv(2);
    // bool		onl1,onl2;
    // long		n=0;

    //(1)
    line_ptr_list::iterator	i,j;
    int				icount,jcount;
    for(j=slaps.begin(),jcount=0;j!=slaps.end();++j,++jcount)
      {
	line_ptr reall2;

	reall2 = (*j);
	reallen2=reall2->length();
	if (reallen2<min_len)
	  continue;

	for(i=slaps.begin(),icount=0;i!=slaps.end() && icount<jcount;
	    ++i,++icount)
	  {
	    line_ptr l1;
	    line_ptr l2 = reall2;
	    double	len2=reallen2;

	    l1 = (*i);

	    //(1.5)
	    len1 = l1->length();
	    if (len1 < min_len )
	      continue;
	    if (l1==l2)	continue;

	    //(2)
	    //calculate the intersection angle using the old method
	    //slow but ok for models. could improve...
	    cross = l1->getIntersectionAngle( *l2, &a );
	
	    if ((double)a<min_angle_diff && (double)a>-min_angle_diff)
	      continue;
	    if ((double)a<-max_angle_diff || (double)a>max_angle_diff)
	      continue;


	    linepair_ptr lp( new linepair );

	    if (a<0)
	      {
		//swap the lines about
		line_ptr t = l1;	double	dt=len1;
		l1=l2;
		l2=t;
		len1=len2;
		len2=dt;
		a = angle((double)a*-1);

	      }

	    //'a' should have the range 0-M_PI now
	    lp->linfo1 = pol_line_info_ptr( new pol_line_info );
	    lp->linfo2 = pol_line_info_ptr( new pol_line_info );

	    lp->linfo1->line = l1;
	    lp->linfo1->len  = len1;
	    lp->linfo1->unitv= unit( l1->getVector() );
	    lp->linfo1->midpoint=l1->findMidPoint();

	    lp->linfo2->line = l2;
	    lp->linfo2->len  = len2;
	    lp->linfo2->unitv= unit( l2->getVector() );
	    lp->linfo2->midpoint=l2->findMidPoint();

	    lp->intersection=cross;

	    //(3) find the distances from the intersection to the start of the lines
	    tv = l1->findMidPoint() - cross;
	    t  = norm_2( tv ) - (len1/2);
	    lp->dist1 = t;

	    tv = l2->findMidPoint() - cross;
	    t  = norm_2( tv ) - (len2/2);
	    lp->dist2 = t;

	    da = (double)a/M_PI;
	    da*=(double)num_abins;
	    abin = (int)da;
	    if (abin>=0 && abin<num_abins)
	      {
		hash_bin[abin].push_back(lp);
		num_lps++;
	      }
	    // else
	    // delete lp;
	  }
      }

    //set the location threshold based on the number of pairs
    //this will change if the other parameters are changed. value
    // here is based on num_a_bins=32, min_len=4.0
    //a bit of a hack,not firmly based...
    loc_thres = pow((double)num_lps, 0.3333);
    loc_thres*=3;
  }


  void	pol_model::match( pol_locator& locate,
			  const pol_line_info &sl1_orig,
			  const pol_line_info &sl2_orig )
  {
    //check if the two lines match to the model and make entry(s)
    //if they do
    bool		done_calc=false;
    double		slen1,sdist1,slen2,sdist2;
    Vector scross( 2 );
    // bool		onsl1,onsl2;
    Vector tv( 2 );
    double		t;
    angle	a;
    int		abin;
    const pol_line_info *sl1 = &sl1_orig, *sl2 = &sl2_orig;
    double		min_s=1.0, max_s=1.0;

    //(0) don't check the lengths, done already

    //(1) check the angle between the two lines
    scross = getIntersectionAngle( *sl1, *sl2, &a );


    //check the angles are > some minimums. we have problems with
    //parallel lines
    if ((double)a<min_angle_diff && (double)a>-min_angle_diff)
      return;
    if ((double)a<-max_angle_diff || (double)a>max_angle_diff)
      return;

    if (a<0)
      {
	//swap the lines about
	const pol_line_info	*t = sl1;
	sl1=sl2;
	sl2=t;
	a = angle((double)a*-1);
      }

    abin = (int)((double)a*(double)num_abins/M_PI);
    if (abin>=0 && abin<num_abins)
      ;//carry on
    else
      return;

    //(2) for each of the lines in the hash bin associated with this
    //angle
    linepair_ptr_list::iterator	lpi;
    for(lpi=hash_bin[abin].begin();lpi!=hash_bin[abin].end();++lpi)
      {
	linepair_ptr lp( *lpi );

	//ok, we have a possible match. now we need to calculate a
	//few more things about our input lines
	if (!done_calc)
	  {
	    done_calc=true;
	    slen1=sl1->len;
	    slen2=sl2->len;

	    tv = sl1->midpoint - scross;
	    t  = norm_2( tv ) - (slen1/2);
	    sdist1 = t;

	    tv = sl2->midpoint - scross;
	    t  = norm_2( tv ) - (slen2/2);
	    sdist2 = t;
	  }

	//if we are looking for scale inv. we have more lines to consider
	//20030222 stumeikle
	if (!scale_inv)
	  {
	    //throw away scene lines which are too long
	    if (slen1>(lp->linfo1->len+len_error) ||
		slen2>(lp->linfo2->len+len_error))
	      continue;

	    //check the distances to see if there is any conceivable match
	    if (sdist1<(lp->dist1-len_error) || sdist2<(lp->dist2-len_error))
	      continue;

	    //fix->possible to simplify this calc by improving the choice of model vars
	    if (sdist1>(lp->dist1+lp->linfo1->len+len_error) ||
		sdist2>(lp->dist2+lp->linfo2->len+len_error)   )
	      continue;
	  }
	else
	  {
	    // we have new criteria for consistence-with-model
	    // in the case of scale invariance...
	    // the scene lines can be consistent with a range of
	    // scales of the model...
	    double	scale_a1_max, scale_a1_min, scale_b1_max, scale_b1_min;

	    // (1) calculate range of consistent scales from the first
	    // line in the pair of scene lines
	    double  btm = lp->dist1;

	    if (btm<len_error)
	      {
		scale_a1_max=scale_b1_max=1000;
	      }
	    else
	      {
		scale_a1_max = sdist1 / (lp->dist1-len_error);
		scale_b1_max = (sdist1+slen1) / (lp->dist1-len_error);
	      }

	    scale_a1_min = sdist1 /(lp->dist1+lp->linfo1->len+len_error);
	    scale_b1_min = (sdist1+slen1) / (lp->dist1+lp->linfo1->len+len_error);

	    //cout << "Scale values are: " << scale_a1_min << " to " << scale_a1_max << endl;
	    //cout << "                : " << scale_b1_min << " to " << scale_b1_max << endl;

	    if (scale_b1_min > scale_a1_min && scale_b1_min<scale_a1_max)
	      {
		//if the first line could conceivably match, then check the second
		//one
		double	scale_a2_max, scale_a2_min, scale_b2_max, scale_b2_min;

		// (2) and do the same for the second line of the pair of lines
		if (lp->dist2<len_error)
		  {
		    scale_a2_max= scale_b2_max=1000;
		  }
		else
		  {
		    scale_a2_max = sdist2 /(lp->dist2-len_error);
		    scale_b2_max = (sdist2+slen1) / (lp->dist2-len_error);
		  }

		scale_a2_min = sdist2 /(lp->dist2+lp->linfo2->len+len_error);
		scale_b2_min = (sdist2+slen1) / (lp->dist2+lp->linfo2->len+len_error);

		if (scale_b2_min > scale_a2_min && scale_b2_min<scale_a2_max)
		  {
		    // (3) check if there's mutual consistency....
		    //cout << "Scale values*are: " << scale_a2_min << " to " << scale_a2_max << endl;
		    //cout << "            *   : " << scale_b2_min << " to " << scale_b2_max << endl;

		    double scale_min1 = scale_b1_min;
		    double scale_max1 = scale_a1_max;
		    double scale_min2 = scale_b2_min;
		    double scale_max2 = scale_a2_max;
		    double min=-1,max=-1;

		    //four cases now
		    if (scale_min1 > scale_min2 && scale_min1 < scale_max2)
		      {
			min = scale_min1;
			if (scale_max1 > scale_max2)
			  max = scale_max2;
			else
			  max = scale_max1;
		      } else if (scale_max1 > scale_min2 && scale_max1 < scale_max2)
			{
			  max = scale_max1;
			  if (scale_min1 < scale_min2)
			    min= scale_min2;
			  else
			    min= scale_min1;
			} else if (scale_min1 < scale_min2 && scale_max1 > scale_max2)
			  {
			    min=scale_min2;
			    max=scale_max2;
			  }

		    if (min!=-1 && max!=-1)
		      {
			//	cout << "              min :" << min << "   and max   " << max << endl;
			//	char temp[10];
			//	cin >> temp;
			max_s=max;
			min_s=min;

			if (min_s>max_scale || max_s<min_scale)
			  continue;

			if (min_s<min_scale)
			  min_s= min_scale;
			if (max_s>max_scale)
			  max_s=max_scale;
		      }
		    else
		      continue;
		  }
		else
		  continue;
	      }
	    else
	      continue;
	  }

	//ok, we got a match------------------------------
	//make an entry into the HT
	//we will try to use the point of intersection here, though i think this
	//may end up being highly inaccurate (error prop to 1/cos(a))
	//but it will reduce the noise from other lines
	angle		angle_diff,angle_diff2;
	Vector centroid(2),dummy(2),dummy2(2);

	//(?) we could make two entries , one per line, but the lines should both produce
	//the same angle_diff result anyway (within errror bounds)
	//use get intersection again as its a better angle estimator
	Vector v1(2),v2(2);

	v1 = lp->linfo1->midpoint - lp->intersection;
	v2 = sl1->midpoint - scross;
	angle_diff = getAngle( v2, v1 );

	centroid = rotate( lp->intersection, -angle_diff );//3secs

	//fix-> weight the points depending on the known errors (angle, sl length
	//	vs model line length)
	//xif<- with weighting doesn't appear to make much difference

	double		weight;

	//weight based on the lengths of the scene lines compared to the
	//model lines. seems to help quite a bit
	weight = slen1*slen2 / ( lp->linfo1->len * lp->linfo2->len );
	if (weight>1)
	  weight=1;

	//locate.addGaussPointFast( scross - centroid, weight, angle(-angle_diff) );
	Vector v( scross - centroid );

	locate.addEntry(weight, v(0), v(1),
			(double) angle_diff,
			scross(0), scross(1), min_s, max_s );
      }
  }


  pol_model::Vector pol_model::getIntersectionAngle
  ( const pol_line_info &l1, const pol_line_info &l2,
    angle *retval)
  {
    //find the intersection of the current line with line l2
    //return the angle between the two in 'retval' and the point of intersection
    //at the usual returned value

    //this is the SAME as the algo in line.cc but faster because we've
    //already calculated the unit vectors etc...
    // bool	b1,b2;
    // double 	d1,d2
    double a;
    Vector v1(2),v2(2),cp(2),b(2),isct(2);
    Vector p( l1.line->getStart() );
    Vector q( l2.line->getStart() );

    v1 = l1.unitv;
    v2 = l2.unitv;

    cp(0) = -v2(1);
    cp(1) =  v2(0);

    a = inner_prod( cp, v1 );

    if (fabs(a)< 0.001) a = copysign(0.001,a);

    a = inner_prod( cp, p-q )/a;
    b = v1*a;

    isct = p - b;

    //now we have the it itsc point, find the angle between the lines
    //result is -M_PI to +M_PI, +ve being anticlockwise on xy plane
    v1 = l1.midpoint;    	v2 = l2.midpoint;
    v1 = v1-isct;		v2 = v2-isct;

    *retval = getAngle( v1, v2 );

    return(isct);
  }


  line_ptr_list pol_model::getModelLines()
  {
    return model_lines;
  }
}
