#include "x11_display.h"

#ifdef HAVE_OPENGL

#include "timer.h"

using namespace std;

namespace mimas {

x11_display::x11_display( const std::string &name ) throw (mimasexception):
  doQuit( false )
{
  display = XOpenDisplay( name.empty() ? (const char *)NULL : name.c_str() );
  MMERROR( display != NULL, mimasexception, , "Error opening X11-display "
           << ( name.empty() ? "$DISPLAY" : name.c_str() ) << "." );
}

void x11_display::event_loop( int timeout, long mask )
  throw (mimasexception)
{

  doQuit = false;
  
  if ( timeout < INT_MAX ) {

    timer t;

    // Remaining time in milliseconds.
    int usecs_remaining;

    do {

      // Check for remaining time.
      usecs_remaining = (int)( ( timeout * 0.001 - t.elapsed() ) * 1E+6 );

      if ( usecs_remaining > 0 ) {
	// Initialise timeout-variable.
	struct timeval tval;
	tval.tv_usec = usecs_remaining % 1000000;
	tval.tv_sec  = usecs_remaining / 1000000;
	
	// Get file-descriptor to event-pipe.
	int fd = ConnectionNumber( display );
	
	// Create file-descriptor set.
	fd_set fds;
	FD_SET( fd, &fds );
	
	// Perform limited wait on event-pipe.
	select( fd + 1, &fds, NULL, NULL, &tval );
      };

      // Empty event-pipe.
      process_events( mask );

    } while ( !doQuit && usecs_remaining > 0 );

  } else {

    // No time-out.

    while ( !doQuit ) {

      XEvent event;

      // Wait for events.
      XMaskEvent( display, mask, &event );

      // Process the event.
      handle_event( event );

    };
  };

}

void x11_display::process_events( long mask ) throw (mimasexception)
{
  // Check for pending events.
  XEvent event;
  while ( XCheckMaskEvent( display, mask, &event ) ) {
    handle_event( event );
  };
}

void x11_display::handle_event( XEvent &event ) throw (mimasexception)
{
  // Find affected window.
  map< Window, x11_window * >::iterator entry =
    windows.find( event.xany.window );
  
  // Forward event.
  if ( entry != windows.end() )
    entry->second->handle_event( event );
}

void x11_display::add_window( x11_window *win )
{
  windows[ win->get_window() ] = win;
}

void x11_display::remove_window( x11_window *win )
{
  windows.erase( win->get_window() );
}


};

#endif

